@@ -54,7 +54,7 @@ public class ModelUtilsTest
 
         ModelUtils.mergePluginDefinitions( plugin, mgtPlugin, false );
 
-        assertEquals( dep.getVersion(), ((Dependency) plugin.getDependencies().get( 0 ) ).getVersion() );
+        assertEquals( dep.getVersion(), plugin.getDependencies().get( 0 ).getVersion() );
     }
 
     private Dependency createDependency( String gid,
@@ -157,9 +157,9 @@ public class ModelUtilsTest
 
         if( configuration != null )
         {
-            for ( Iterator it = configuration.entrySet().iterator(); it.hasNext(); )
+            for ( Object o : configuration.entrySet() )
             {
-                Map.Entry entry = (Map.Entry) it.next();
+                Map.Entry entry = (Map.Entry) o;
 
                 Xpp3Dom param = new Xpp3Dom( String.valueOf( entry.getKey() ) );
                 param.setValue( String.valueOf( entry.getValue() ) );
@@ -432,7 +432,7 @@ public class ModelUtilsTest
 
         // this block prevents MNG-1703
         assertEquals( 1, child.getDependencies().size() );
-        Dependency dep2 = (Dependency) child.getDependencies().get( 0 );
+        Dependency dep2 = child.getDependencies().get( 0 );
         assertEquals( dep.getManagementKey(), dep2.getManagementKey() );
     }
 
