@@ -76,7 +76,7 @@ public class DefaultArtifactCollectorTest
 
         source = new Source();
         artifactFactory = (ArtifactFactory) lookup( ArtifactFactory.ROLE );
-        artifactCollector = (LegacyArtifactCollector) lookup( LegacyArtifactCollector.class );
+        artifactCollector = lookup( LegacyArtifactCollector.class );
 
         projectArtifact = createArtifactSpec( "project", "1.0", null );
     }
@@ -712,16 +712,16 @@ public class DefaultArtifactCollectorTest
         }
         catch ( OverConstrainedVersionException e )
         {
-            assertTrue( "Versions unordered", e.getMessage().indexOf( "[3.2.1-v3235e, 3.3.0-v3346]" ) != -1 );
-            assertTrue( "DependencyTrail unresolved", e.getMessage().indexOf( "Path to dependency:" ) != -1 );
+            assertTrue( "Versions unordered", e.getMessage().contains( "[3.2.1-v3235e, 3.3.0-v3346]" ) );
+            assertTrue( "DependencyTrail unresolved", e.getMessage().contains( "Path to dependency:" ) );
         }
     }
 
     private Artifact getArtifact( String id, Set artifacts )
     {
-        for ( Iterator i = artifacts.iterator(); i.hasNext(); )
+        for ( Object artifact : artifacts )
         {
-            Artifact a = (Artifact) i.next();
+            Artifact a = (Artifact) artifact;
             if ( a.getArtifactId().equals( id ) && a.getGroupId().equals( GROUP_ID ) )
             {
                 return a;
@@ -886,9 +886,9 @@ public class DefaultArtifactCollectorTest
         {
             Set projectArtifacts = new HashSet();
 
-            for ( Iterator i = dependencies.iterator(); i.hasNext(); )
+            for ( Object dependency : dependencies )
             {
-                Artifact d = (Artifact) i.next();
+                Artifact d = (Artifact) dependency;
 
                 VersionRange versionRange;
                 if ( d.getVersionRange() != null )
@@ -904,8 +904,8 @@ public class DefaultArtifactCollectorTest
                 {
                     /* don't call createDependencyArtifact as it'll ignore test and provided scopes */
                     artifact =
-                        artifactFactory.createArtifact( d.getGroupId(), d.getArtifactId(), d.getVersion(),
-                                                        d.getScope(), d.getType() );
+                        artifactFactory.createArtifact( d.getGroupId(), d.getArtifactId(), d.getVersion(), d.getScope(),
+                                                        d.getType() );
                 }
                 else
                 {
