@@ -1,38 +1,42 @@
-/*
- * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
- * agreements. See the NOTICE file distributed with this work for additional information regarding
- * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance with the License. You may obtain a
- * copy of the License at
- *
- * http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software distributed under the License
- * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
- * or implied. See the License for the specific language governing permissions and limitations under
- * the License.
- */
 package org.apache.maven.lifecycle.internal;
 
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *  http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.TreeMap;
 
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.lifecycle.DefaultLifecycles;
-import org.apache.maven.lifecycle.DefaultSchedules;
 import org.apache.maven.lifecycle.Lifecycle;
+import org.apache.maven.lifecycle.LifecycleMappingDelegate;
 import org.apache.maven.lifecycle.LifecycleNotFoundException;
 import org.apache.maven.lifecycle.LifecyclePhaseNotFoundException;
 import org.apache.maven.lifecycle.MavenExecutionPlan;
-import org.apache.maven.model.Plugin;
-import org.apache.maven.model.PluginExecution;
+import org.apache.maven.lifecycle.MojoExecutionConfigurator;
+import org.apache.maven.lifecycle.internal.builder.BuilderCommon;
 import org.apache.maven.plugin.BuildPluginManager;
 import org.apache.maven.plugin.InvalidPluginDescriptorException;
 import org.apache.maven.plugin.MojoExecution;
@@ -55,6 +59,8 @@ import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 
+import com.google.common.collect.ImmutableMap;
+
 /**
  * @since 3.0
  * @author Benjamin Bentmann
@@ -75,34 +81,43 @@ public class DefaultLifecycleExecutionPlanCalculator
     @Requirement
     private DefaultLifecycles defaultLifeCycles;
 
-    @Requirement
-    private DefaultSchedules defaultSchedules;
-
     @Requirement
     private MojoDescriptorCreator mojoDescriptorCreator;
 
     @Requirement
     private LifecyclePluginResolver lifecyclePluginResolver;
 
+    @Requirement( hint = DefaultLifecycleMappingDelegate.HINT )
+    private LifecycleMappingDelegate standardDelegate;
+
+    @Requirement
+    private Map<String, LifecycleMappingDelegate> delegates;
+
+    @Requirement
+    private Map<String, MojoExecutionConfigurator> mojoExecutionConfigurators;
+
     @SuppressWarnings( { "UnusedDeclaration" } )
     public DefaultLifecycleExecutionPlanCalculator()
     {
     }
 
+    // Only used for testing
     public DefaultLifecycleExecutionPlanCalculator( BuildPluginManager pluginManager,
                                                     DefaultLifecycles defaultLifeCycles,
                                                     MojoDescriptorCreator mojoDescriptorCreator,
-                                                    LifecyclePluginResolver lifecyclePluginResolver,
-                                                    DefaultSchedules defaultSchedules )
+                                                    LifecyclePluginResolver lifecyclePluginResolver )
     {
         this.pluginManager = pluginManager;
         this.defaultLifeCycles = defaultLifeCycles;
         this.mojoDescriptorCreator = mojoDescriptorCreator;
         this.lifecyclePluginResolver = lifecyclePluginResolver;
-        this.defaultSchedules = defaultSchedules;
+        this.mojoExecutionConfigurators =
+            ImmutableMap.of( "default", (MojoExecutionConfigurator) new DefaultMojoExecutionConfigurator() );
     }
 
-    public MavenExecutionPlan calculateExecutionPlan( MavenSession session, MavenProject project, List<Object> tasks, boolean setup )
+    @Override
+    public MavenExecutionPlan calculateExecutionPlan( MavenSession session, MavenProject project, List<Object> tasks,
+                                                      boolean setup )
         throws PluginNotFoundException, PluginResolutionException, LifecyclePhaseNotFoundException,
         PluginDescriptorParsingException, MojoNotFoundException, InvalidPluginDescriptorException,
         NoPluginFoundForPrefixException, LifecycleNotFoundException, PluginVersionResolutionException
@@ -116,11 +131,12 @@ public class DefaultLifecycleExecutionPlanCalculator
             setupMojoExecutions( session, project, executions );
         }
 
-        final List<ExecutionPlanItem> planItem = defaultSchedules.createExecutionPlanItem( project, executions );
+        final List<ExecutionPlanItem> planItem = ExecutionPlanItem.createExecutionPlanItems( project, executions );
 
         return new MavenExecutionPlan( planItem, defaultLifeCycles );
     }
 
+    @Override
     public MavenExecutionPlan calculateExecutionPlan( MavenSession session, MavenProject project, List<Object> tasks )
         throws PluginNotFoundException, PluginResolutionException, LifecyclePhaseNotFoundException,
         PluginDescriptorParsingException, MojoNotFoundException, InvalidPluginDescriptorException,
@@ -140,6 +156,7 @@ public class DefaultLifecycleExecutionPlanCalculator
         }
     }
 
+    @Override
     public void setupMojoExecution( MavenSession session, MavenProject project, MojoExecution mojoExecution )
         throws PluginNotFoundException, PluginResolutionException, PluginDescriptorParsingException,
         MojoNotFoundException, InvalidPluginDescriptorException, NoPluginFoundForPrefixException,
@@ -157,16 +174,16 @@ public class DefaultLifecycleExecutionPlanCalculator
             mojoExecution.setMojoDescriptor( mojoDescriptor );
         }
 
-        populateMojoExecutionConfiguration( project, mojoExecution,
-                                            MojoExecution.Source.CLI.equals( mojoExecution.getSource() ) );
+        mojoExecutionConfigurator( mojoExecution ).configure( project,
+                                                              mojoExecution,
+                                                        MojoExecution.Source.CLI.equals( mojoExecution.getSource() ) );
 
         finalizeMojoConfiguration( mojoExecution );
 
         calculateForkedExecutions( mojoExecution, session, project, new HashSet<MojoDescriptor>() );
     }
-    
-    public List<MojoExecution> calculateMojoExecutions( MavenSession session, MavenProject project,
-                                                         List<Object> tasks )
+
+    public List<MojoExecution> calculateMojoExecutions( MavenSession session, MavenProject project, List<Object> tasks )
         throws PluginNotFoundException, PluginResolutionException, PluginDescriptorParsingException,
         MojoNotFoundException, NoPluginFoundForPrefixException, InvalidPluginDescriptorException,
         PluginVersionResolutionException, LifecyclePhaseNotFoundException
@@ -179,10 +196,17 @@ public class DefaultLifecycleExecutionPlanCalculator
             {
                 String pluginGoal = ( (GoalTask) task ).pluginGoal;
 
+                String executionId = "default-cli";
+                int executionIdx = pluginGoal.indexOf( '@' );
+                if ( executionIdx > 0 )
+                {
+                    executionId = pluginGoal.substring( executionIdx + 1 );
+                }
+
                 MojoDescriptor mojoDescriptor = mojoDescriptorCreator.getMojoDescriptor( pluginGoal, session, project );
 
-                MojoExecution mojoExecution =
-                    new MojoExecution( mojoDescriptor, "default-cli", MojoExecution.Source.CLI );
+                MojoExecution mojoExecution = new MojoExecution( mojoDescriptor, executionId,
+                                                                 MojoExecution.Source.CLI );
 
                 mojoExecutions.add( mojoExecution );
             }
@@ -219,176 +243,27 @@ public class DefaultLifecycleExecutionPlanCalculator
 
         if ( lifecycle == null )
         {
-            throw new LifecyclePhaseNotFoundException(
-                "Unknown lifecycle phase \"" + lifecyclePhase + "\". You must specify a valid lifecycle phase" +
-                    " or a goal in the format <plugin-prefix>:<goal> or" +
-                    " <plugin-group-id>:<plugin-artifact-id>[:<plugin-version>]:<goal>. Available lifecycle phases are: " +
-                    defaultLifeCycles.getLifecyclePhaseList() + ".", lifecyclePhase );
+            throw new LifecyclePhaseNotFoundException( "Unknown lifecycle phase \"" + lifecyclePhase
+                + "\". You must specify a valid lifecycle phase" + " or a goal in the format <plugin-prefix>:<goal> or"
+                + " <plugin-group-id>:<plugin-artifact-id>[:<plugin-version>]:<goal>. Available lifecycle phases are: "
+                + defaultLifeCycles.getLifecyclePhaseList() + ".", lifecyclePhase );
         }
 
-        /*
-         * Initialize mapping from lifecycle phase to bound mojos. The key set of this map denotes the phases the caller
-         * is interested in, i.e. all phases up to and including the specified phase.
-         */
-
-        Map<String, Map<Integer, List<MojoExecution>>> mappings =
-            new LinkedHashMap<String, Map<Integer, List<MojoExecution>>>();
-
-        for ( String phase : lifecycle.getPhases() )
+        LifecycleMappingDelegate delegate;
+        if ( Arrays.binarySearch( DefaultLifecycles.STANDARD_LIFECYCLES, lifecycle.getId() ) >= 0 )
         {
-            Map<Integer, List<MojoExecution>> phaseBindings = new TreeMap<Integer, List<MojoExecution>>();
-
-            mappings.put( phase, phaseBindings );
-
-            if ( phase.equals( lifecyclePhase ) )
+            delegate = standardDelegate;
+        }
+        else
+        {
+            delegate = delegates.get( lifecycle.getId() );
+            if ( delegate == null )
             {
-                break;
+                delegate = standardDelegate;
             }
         }
 
-        /*
-         * Grab plugin executions that are bound to the selected lifecycle phases from project. The effective model of
-         * the project already contains the plugin executions induced by the project's packaging type. Remember, all
-         * phases of interest and only those are in the lifecyle mapping, if a phase has no value in the map, we are not
-         * interested in any of the executions bound to it.
-         */
-
-        for ( Plugin plugin : project.getBuild().getPlugins() )
-        {
-            for ( PluginExecution execution : plugin.getExecutions() )
-            {
-                // if the phase is specified then I don't have to go fetch the plugin yet and pull it down
-                // to examine the phase it is associated to.
-                if ( execution.getPhase() != null )
-                {
-                    Map<Integer, List<MojoExecution>> phaseBindings = mappings.get( execution.getPhase() );
-                    if ( phaseBindings != null )
-                    {
-                        for ( String goal : execution.getGoals() )
-                        {
-                            MojoExecution mojoExecution = new MojoExecution( plugin, goal, execution.getId() );
-                            mojoExecution.setLifecyclePhase( execution.getPhase() );
-                            addMojoExecution( phaseBindings, mojoExecution, execution.getPriority() );
-                        }
-                    }
-                }
-                // if not then i need to grab the mojo descriptor and look at the phase that is specified
-                else
-                {
-                    for ( String goal : execution.getGoals() )
-                    {
-                        MojoDescriptor mojoDescriptor =
-                            pluginManager.getMojoDescriptor( plugin, goal, project.getRemotePluginRepositories(),
-                                                             session.getRepositorySession() );
-
-                        Map<Integer, List<MojoExecution>> phaseBindings = mappings.get( mojoDescriptor.getPhase() );
-                        if ( phaseBindings != null )
-                        {
-                            MojoExecution mojoExecution = new MojoExecution( mojoDescriptor, execution.getId() );
-                            mojoExecution.setLifecyclePhase( mojoDescriptor.getPhase() );
-                            addMojoExecution( phaseBindings, mojoExecution, execution.getPriority() );
-                        }
-                    }
-                }
-            }
-        }
-
-        Map<String, List<MojoExecution>> lifecycleMappings = new LinkedHashMap<String, List<MojoExecution>>();
-
-        for ( Map.Entry<String, Map<Integer, List<MojoExecution>>> entry : mappings.entrySet() )
-        {
-            List<MojoExecution> mojoExecutions = new ArrayList<MojoExecution>();
-
-            for ( List<MojoExecution> executions : entry.getValue().values() )
-            {
-                mojoExecutions.addAll( executions );
-            }
-
-            lifecycleMappings.put( entry.getKey(), mojoExecutions );
-        }
-
-        return lifecycleMappings;
-    }
-
-    private void addMojoExecution( Map<Integer, List<MojoExecution>> phaseBindings, MojoExecution mojoExecution,
-                                   int priority )
-    {
-        List<MojoExecution> mojoExecutions = phaseBindings.get( priority );
-
-        if ( mojoExecutions == null )
-        {
-            mojoExecutions = new ArrayList<MojoExecution>();
-            phaseBindings.put( priority, mojoExecutions );
-        }
-
-        mojoExecutions.add( mojoExecution );
-    }
-
-    private void populateMojoExecutionConfiguration( MavenProject project, MojoExecution mojoExecution,
-                                                     boolean allowPluginLevelConfig )
-    {
-        String g = mojoExecution.getGroupId();
-
-        String a = mojoExecution.getArtifactId();
-
-        Plugin plugin = findPlugin( g, a, project.getBuildPlugins() );
-
-        if ( plugin == null && project.getPluginManagement() != null )
-        {
-            plugin = findPlugin( g, a, project.getPluginManagement().getPlugins() );
-        }
-
-        if ( plugin != null )
-        {
-            PluginExecution pluginExecution =
-                findPluginExecution( mojoExecution.getExecutionId(), plugin.getExecutions() );
-
-            Xpp3Dom pomConfiguration = null;
-
-            if ( pluginExecution != null )
-            {
-                pomConfiguration = (Xpp3Dom) pluginExecution.getConfiguration();
-            }
-            else if ( allowPluginLevelConfig )
-            {
-                pomConfiguration = (Xpp3Dom) plugin.getConfiguration();
-            }
-
-            Xpp3Dom mojoConfiguration = ( pomConfiguration != null ) ? new Xpp3Dom( pomConfiguration ) : null;
-
-            mojoConfiguration = Xpp3Dom.mergeXpp3Dom( mojoExecution.getConfiguration(), mojoConfiguration );
-
-            mojoExecution.setConfiguration( mojoConfiguration );
-        }
-    }
-
-    private Plugin findPlugin( String groupId, String artifactId, Collection<Plugin> plugins )
-    {
-        for ( Plugin plugin : plugins )
-        {
-            if ( artifactId.equals( plugin.getArtifactId() ) && groupId.equals( plugin.getGroupId() ) )
-            {
-                return plugin;
-            }
-        }
-
-        return null;
-    }
-
-    private PluginExecution findPluginExecution( String executionId, Collection<PluginExecution> executions )
-    {
-        if ( StringUtils.isNotEmpty( executionId ) )
-        {
-            for ( PluginExecution execution : executions )
-            {
-                if ( executionId.equals( execution.getId() ) )
-                {
-                    return execution;
-                }
-            }
-        }
-
-        return null;
+        return delegate.calculateLifecycleMappings( session, project, lifecycle, lifecyclePhase );
     }
 
     /**
@@ -425,15 +300,15 @@ public class DefaultLifecycleExecutionPlanCalculator
 
                 Xpp3Dom parameterDefaults = defaultConfiguration.getChild( parameter.getName() );
 
-                parameterConfiguration =
-                    Xpp3Dom.mergeXpp3Dom( parameterConfiguration, parameterDefaults, Boolean.TRUE );
+                parameterConfiguration = Xpp3Dom.mergeXpp3Dom( parameterConfiguration, parameterDefaults,
+                                                               Boolean.TRUE );
 
                 if ( parameterConfiguration != null )
                 {
                     parameterConfiguration = new Xpp3Dom( parameterConfiguration, parameter.getName() );
 
-                    if ( StringUtils.isEmpty( parameterConfiguration.getAttribute( "implementation" ) ) &&
-                        StringUtils.isNotEmpty( parameter.getImplementation() ) )
+                    if ( StringUtils.isEmpty( parameterConfiguration.getAttribute( "implementation" ) )
+                        && StringUtils.isNotEmpty( parameter.getImplementation() ) )
                     {
                         parameterConfiguration.setAttribute( "implementation", parameter.getImplementation() );
                     }
@@ -451,6 +326,7 @@ public class DefaultLifecycleExecutionPlanCalculator
         return MojoDescriptorCreator.convert( mojoDescriptor );
     }
 
+    @Override
     public void calculateForkedExecutions( MojoExecution mojoExecution, MavenSession session )
         throws MojoNotFoundException, PluginNotFoundException, PluginResolutionException,
         PluginDescriptorParsingException, NoPluginFoundForPrefixException, InvalidPluginDescriptorException,
@@ -496,8 +372,8 @@ public class DefaultLifecycleExecutionPlanCalculator
             }
             else
             {
-                forkedExecutions =
-                    calculateForkedGoal( mojoExecution, session, forkedProject, alreadyForkedExecutions );
+                forkedExecutions = calculateForkedGoal( mojoExecution, session, forkedProject,
+                                                        alreadyForkedExecutions );
             }
 
             mojoExecution.setForkedExecutions( BuilderCommon.getKey( forkedProject ), forkedExecutions );
@@ -517,8 +393,8 @@ public class DefaultLifecycleExecutionPlanCalculator
 
         String forkedPhase = mojoDescriptor.getExecutePhase();
 
-        Map<String, List<MojoExecution>> lifecycleMappings =
-            calculateLifecycleMappings( session, project, forkedPhase );
+        Map<String, List<MojoExecution>> lifecycleMappings = calculateLifecycleMappings( session, project,
+                                                                                         forkedPhase );
 
         for ( List<MojoExecution> forkedExecutions : lifecycleMappings.values() )
         {
@@ -534,7 +410,7 @@ public class DefaultLifecycleExecutionPlanCalculator
                     forkedExecution.setMojoDescriptor( forkedMojoDescriptor );
                 }
 
-                populateMojoExecutionConfiguration( project, forkedExecution, false );
+                mojoExecutionConfigurator( forkedExecution ).configure( project, forkedExecution, false );
             }
         }
 
@@ -629,7 +505,7 @@ public class DefaultLifecycleExecutionPlanCalculator
 
                         forkedExecution.setConfiguration( forkedConfiguration );
 
-                        populateMojoExecutionConfiguration( project, forkedExecution, true );
+                        mojoExecutionConfigurator( forkedExecution ).configure( project, forkedExecution, true );
 
                         forkedExecutions.add( forkedExecution );
                     }
@@ -651,10 +527,11 @@ public class DefaultLifecycleExecutionPlanCalculator
             }
         }
     }
+
     // org.apache.maven.plugins:maven-remote-resources-plugin:1.0:process
-    //TODO: take repo mans into account as one may be aggregating prefixes of many
-    //TODO: collect at the root of the repository, read the one at the root, and fetch remote if something is missing
-    //      or the user forces the issue
+    // TODO: take repo mans into account as one may be aggregating prefixes of many
+    // TODO: collect at the root of the repository, read the one at the root, and fetch remote if something is missing
+    // or the user forces the issue
 
     private List<MojoExecution> calculateForkedGoal( MojoExecution mojoExecution, MavenSession session,
                                                      MavenProject project,
@@ -682,7 +559,7 @@ public class DefaultLifecycleExecutionPlanCalculator
 
         MojoExecution forkedExecution = new MojoExecution( forkedMojoDescriptor, forkedGoal );
 
-        populateMojoExecutionConfiguration( project, forkedExecution, true );
+        mojoExecutionConfigurator( forkedExecution ).configure( project, forkedExecution, true );
 
         finalizeMojoConfiguration( forkedExecution );
 
@@ -691,5 +568,24 @@ public class DefaultLifecycleExecutionPlanCalculator
         return Collections.singletonList( forkedExecution );
     }
 
+    private MojoExecutionConfigurator mojoExecutionConfigurator( MojoExecution mojoExecution )
+    {
+        String configuratorId = mojoExecution.getMojoDescriptor().getComponentConfigurator();
+        if ( configuratorId == null )
+        {
+            configuratorId = "default";
+        }
 
+        MojoExecutionConfigurator mojoExecutionConfigurator = mojoExecutionConfigurators.get( configuratorId );
+
+        if ( mojoExecutionConfigurator == null )
+        {
+            //
+            // The plugin has a custom component configurator but does not have a custom mojo execution configurator
+            // so fall back to the default mojo execution configurator.
+            //
+            mojoExecutionConfigurator = mojoExecutionConfigurators.get( "default" );
+        }
+        return mojoExecutionConfigurator;
+    }
 }
