@@ -1,18 +1,22 @@
 package org.apache.maven.project;
 
 /*
- * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
- * agreements. See the NOTICE file distributed with this work for additional information regarding
- * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance with the License. You may obtain a
- * copy of the License at
- * 
- * http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software distributed under the License
- * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
- * or implied. See the License for the specific language governing permissions and limitations under
- * the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *  http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 import java.io.File;
@@ -21,6 +25,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -28,10 +33,20 @@ import java.util.Set;
 
 import org.apache.maven.RepositoryUtils;
 import org.apache.maven.artifact.Artifact;
+import org.apache.maven.artifact.InvalidRepositoryException;
+import org.apache.maven.artifact.repository.ArtifactRepository;
 import org.apache.maven.artifact.repository.LegacyLocalRepositoryManager;
+import org.apache.maven.bridge.MavenRepositorySystem;
 import org.apache.maven.model.Build;
+import org.apache.maven.model.Dependency;
+import org.apache.maven.model.DependencyManagement;
+import org.apache.maven.model.DeploymentRepository;
+import org.apache.maven.model.Extension;
 import org.apache.maven.model.Model;
+import org.apache.maven.model.Parent;
+import org.apache.maven.model.Plugin;
 import org.apache.maven.model.Profile;
+import org.apache.maven.model.ReportPlugin;
 import org.apache.maven.model.building.DefaultModelBuildingRequest;
 import org.apache.maven.model.building.DefaultModelProblem;
 import org.apache.maven.model.building.FileModelSource;
@@ -44,22 +59,23 @@ import org.apache.maven.model.building.ModelProcessor;
 import org.apache.maven.model.building.ModelSource;
 import org.apache.maven.model.building.StringModelSource;
 import org.apache.maven.model.resolution.ModelResolver;
-import org.apache.maven.repository.RepositorySystem;
 import org.apache.maven.repository.internal.ArtifactDescriptorUtils;
 import org.codehaus.plexus.component.annotations.Component;
 import org.codehaus.plexus.component.annotations.Requirement;
 import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.util.Os;
 import org.codehaus.plexus.util.StringUtils;
-import org.sonatype.aether.RepositorySystemSession;
-import org.sonatype.aether.RequestTrace;
-import org.sonatype.aether.impl.RemoteRepositoryManager;
-import org.sonatype.aether.repository.LocalRepositoryManager;
-import org.sonatype.aether.repository.RemoteRepository;
-import org.sonatype.aether.repository.WorkspaceRepository;
-import org.sonatype.aether.resolution.ArtifactRequest;
-import org.sonatype.aether.resolution.ArtifactResult;
-import org.sonatype.aether.util.DefaultRequestTrace;
+import org.eclipse.aether.RepositorySystemSession;
+import org.eclipse.aether.RequestTrace;
+import org.eclipse.aether.impl.RemoteRepositoryManager;
+import org.eclipse.aether.repository.LocalRepositoryManager;
+import org.eclipse.aether.repository.RemoteRepository;
+import org.eclipse.aether.repository.WorkspaceRepository;
+import org.eclipse.aether.resolution.ArtifactRequest;
+import org.eclipse.aether.resolution.ArtifactResult;
+import org.eclipse.aether.resolution.VersionRangeRequest;
+import org.eclipse.aether.resolution.VersionRangeResolutionException;
+import org.eclipse.aether.resolution.VersionRangeResult;
 
 /**
  */
@@ -81,10 +97,10 @@ public class DefaultProjectBuilder
     private ProjectBuildingHelper projectBuildingHelper;
 
     @Requirement
-    private RepositorySystem repositorySystem;
+    private MavenRepositorySystem repositorySystem;
 
     @Requirement
-    private org.sonatype.aether.RepositorySystem repoSystem;
+    private org.eclipse.aether.RepositorySystem repoSystem;
 
     @Requirement
     private RemoteRepositoryManager repositoryManager;
@@ -96,16 +112,18 @@ public class DefaultProjectBuilder
     // MavenProjectBuilder Implementation
     // ----------------------------------------------------------------------
 
+    @Override
     public ProjectBuildingResult build( File pomFile, ProjectBuildingRequest request )
         throws ProjectBuildingException
     {
-        return build( pomFile, new FileModelSource( pomFile ), new InternalConfig( request, null, null ) );
+        return build( pomFile, new FileModelSource( pomFile ), new InternalConfig( request, null ) );
     }
 
+    @Override
     public ProjectBuildingResult build( ModelSource modelSource, ProjectBuildingRequest request )
         throws ProjectBuildingException
     {
-        return build( null, modelSource, new InternalConfig( request, null, null ) );
+        return build( null, modelSource, new InternalConfig( request, null ) );
     }
 
     private ProjectBuildingResult build( File pomFile, ModelSource modelSource, InternalConfig config )
@@ -115,9 +133,9 @@ public class DefaultProjectBuilder
 
         try
         {
-            ProjectBuildingRequest configuration = config.request;
+            ProjectBuildingRequest projectBuildingRequest = config.request;
 
-            MavenProject project = configuration.getProject();
+            MavenProject project = projectBuildingRequest.getProject();
 
             List<ModelProblem> modelProblems = null;
             Throwable error = null;
@@ -126,10 +144,11 @@ public class DefaultProjectBuilder
             {
                 ModelBuildingRequest request = getModelBuildingRequest( config );
 
-                project = new MavenProject( repositorySystem, this, configuration, logger );
+                project = new MavenProject();
+                project.setFile( pomFile );
 
                 DefaultModelBuildingListener listener =
-                    new DefaultModelBuildingListener( project, projectBuildingHelper, configuration );
+                    new DefaultModelBuildingListener( project, projectBuildingHelper, projectBuildingRequest );
                 request.setModelBuildingListener( listener );
 
                 request.setPomFile( pomFile );
@@ -154,17 +173,17 @@ public class DefaultProjectBuilder
 
                 modelProblems = result.getProblems();
 
-                initProject( project, Collections.<String, MavenProject> emptyMap(), result,
-                             new HashMap<File, Boolean>() );
+                initProject( project, Collections.<String, MavenProject>emptyMap(), result,
+                             new HashMap<File, Boolean>(), projectBuildingRequest );
             }
-            else if ( configuration.isResolveDependencies() )
+            else if ( projectBuildingRequest.isResolveDependencies() )
             {
                 projectBuildingHelper.selectProjectRealm( project );
             }
 
             DependencyResolutionResult resolutionResult = null;
 
-            if ( configuration.isResolveDependencies() )
+            if ( projectBuildingRequest.isResolveDependencies() )
             {
                 resolutionResult = resolveDependencies( project, config.session );
             }
@@ -188,7 +207,7 @@ public class DefaultProjectBuilder
 
     private DependencyResolutionResult resolveDependencies( MavenProject project, RepositorySystemSession session )
     {
-        DependencyResolutionResult resolutionResult = null;
+        DependencyResolutionResult resolutionResult;
 
         try
         {
@@ -241,7 +260,7 @@ public class DefaultProjectBuilder
 
         ModelBuildingRequest request = new DefaultModelBuildingRequest();
 
-        RequestTrace trace = DefaultRequestTrace.newChild( null, configuration ).newChild( request );
+        RequestTrace trace = RequestTrace.newChild( null, configuration ).newChild( request );
 
         ModelResolver resolver =
             new ProjectModelResolver( config.session, trace, repoSystem, repositoryManager, config.repositories,
@@ -256,27 +275,67 @@ public class DefaultProjectBuilder
         request.setUserProperties( configuration.getUserProperties() );
         request.setBuildStartTime( configuration.getBuildStartTime() );
         request.setModelResolver( resolver );
-        request.setModelCache( config.modelCache );
+        request.setModelCache( new ReactorModelCache() );
 
         return request;
     }
 
+    @Override
     public ProjectBuildingResult build( Artifact artifact, ProjectBuildingRequest request )
         throws ProjectBuildingException
     {
         return build( artifact, false, request );
     }
 
+    @Override
     public ProjectBuildingResult build( Artifact artifact, boolean allowStubModel, ProjectBuildingRequest request )
         throws ProjectBuildingException
     {
-        org.sonatype.aether.artifact.Artifact pomArtifact = RepositoryUtils.toArtifact( artifact );
+        org.eclipse.aether.artifact.Artifact pomArtifact = RepositoryUtils.toArtifact( artifact );
         pomArtifact = ArtifactDescriptorUtils.toPomArtifact( pomArtifact );
 
-        InternalConfig config = new InternalConfig( request, null, null );
+        InternalConfig config = new InternalConfig( request, null );
 
         boolean localProject;
 
+        if ( request.isResolveVersionRanges() )
+        {
+            VersionRangeRequest versionRangeRequest = new VersionRangeRequest( pomArtifact, config.repositories, null );
+
+            try
+            {
+                VersionRangeResult versionRangeResult =
+                    repoSystem.resolveVersionRange( config.session, versionRangeRequest );
+
+                if ( versionRangeResult.getHighestVersion() == null )
+                {
+                    throw new ProjectBuildingException(
+                        artifact.getId(), "Error resolving project artifact: No versions matched the requested range",
+                        (Throwable) null );
+
+                }
+
+                if ( versionRangeResult.getVersionConstraint() != null
+                         && versionRangeResult.getVersionConstraint().getRange() != null
+                         && versionRangeResult.getVersionConstraint().getRange().getUpperBound() == null )
+                {
+                    throw new ProjectBuildingException(
+                        artifact.getId(),
+                        "Error resolving project artifact: The requested version range does not specify an upper bound",
+                        (Throwable) null );
+
+                }
+
+                pomArtifact = pomArtifact.setVersion( versionRangeResult.getHighestVersion().toString() );
+            }
+            catch ( VersionRangeResolutionException e )
+            {
+                throw new ProjectBuildingException(
+                    artifact.getId(), "Error resolving project artifact: " + e.getMessage(), e );
+
+            }
+        }
+
         try
         {
             ArtifactRequest pomRequest = new ArtifactRequest();
@@ -287,7 +346,7 @@ public class DefaultProjectBuilder
             pomArtifact = pomResult.getArtifact();
             localProject = pomResult.getRepository() instanceof WorkspaceRepository;
         }
-        catch ( org.sonatype.aether.resolution.ArtifactResolutionException e )
+        catch ( org.eclipse.aether.resolution.ArtifactResolutionException e )
         {
             if ( e.getResults().get( 0 ).isMissing() && allowStubModel )
             {
@@ -325,6 +384,7 @@ public class DefaultProjectBuilder
         return new StringModelSource( buffer, artifact.getId() );
     }
 
+    @Override
     public List<ProjectBuildingResult> build( List<File> pomFiles, boolean recursive, ProjectBuildingRequest request )
         throws ProjectBuildingException
     {
@@ -334,14 +394,13 @@ public class DefaultProjectBuilder
 
         ReactorModelPool modelPool = new ReactorModelPool();
 
-        ReactorModelCache modelCache = new ReactorModelCache();
-
-        InternalConfig config = new InternalConfig( request, modelPool, modelCache );
+        InternalConfig config = new InternalConfig( request, modelPool );
 
         Map<String, MavenProject> projectIndex = new HashMap<String, MavenProject>( 256 );
 
         boolean noErrors =
-            build( results, interimResults, projectIndex, pomFiles, new LinkedHashSet<File>(), true, recursive, config );
+            build( results, interimResults, projectIndex, pomFiles, new LinkedHashSet<File>(), true, recursive,
+                   config );
 
         populateReactorModelPool( modelPool, interimResults );
 
@@ -395,7 +454,7 @@ public class DefaultProjectBuilder
 
         ModelBuildingRequest request = getModelBuildingRequest( config );
 
-        MavenProject project = new MavenProject( repositorySystem, this, config.request, logger );
+        MavenProject project = new MavenProject();
 
         request.setPomFile( pomFile );
         request.setTwoPhaseBuilding( true );
@@ -442,7 +501,8 @@ public class DefaultProjectBuilder
                     {
                         ModelProblem problem =
                             new DefaultModelProblem( "Child module " + moduleFile + " of " + pomFile
-                                + " does not exist", ModelProblem.Severity.ERROR, model, -1, -1, null );
+                                + " does not exist", ModelProblem.Severity.ERROR, ModelProblem.Version.BASE, model, -1,
+                                                     -1, null );
                         result.getProblems().add( problem );
 
                         noErrors = false;
@@ -478,8 +538,8 @@ public class DefaultProjectBuilder
 
                         ModelProblem problem =
                             new DefaultModelProblem( "Child module " + moduleFile + " of " + pomFile
-                                + " forms aggregation cycle " + buffer, ModelProblem.Severity.ERROR, model, -1, -1,
-                                                     null );
+                                + " forms aggregation cycle " + buffer, ModelProblem.Severity.ERROR,
+                                                     ModelProblem.Version.BASE, model, -1, -1, null );
                         result.getProblems().add( problem );
 
                         noErrors = false;
@@ -560,7 +620,7 @@ public class DefaultProjectBuilder
                 ModelBuildingResult result = modelBuilder.build( interimResult.request, interimResult.result );
 
                 MavenProject project = interimResult.listener.getProject();
-                initProject( project, projectIndex, result, profilesXmls );
+                initProject( project, projectIndex, result, profilesXmls, request );
 
                 List<MavenProject> modules = new ArrayList<MavenProject>();
                 noErrors =
@@ -576,7 +636,8 @@ public class DefaultProjectBuilder
             }
             catch ( ModelBuildingException e )
             {
-                results.add( new DefaultProjectBuildingResult( e.getModelId(), interimResult.pomFile, e.getProblems() ) );
+                results.add( new DefaultProjectBuildingResult( e.getModelId(), interimResult.pomFile,
+                                                               e.getProblems() ) );
 
                 noErrors = false;
             }
@@ -586,19 +647,59 @@ public class DefaultProjectBuilder
     }
 
     private void initProject( MavenProject project, Map<String, MavenProject> projects, ModelBuildingResult result,
-                              Map<File, Boolean> profilesXmls )
+                              Map<File, Boolean> profilesXmls, ProjectBuildingRequest projectBuildingRequest )
     {
         Model model = result.getEffectiveModel();
 
         project.setModel( model );
         project.setOriginalModel( result.getRawModel() );
-
         project.setFile( model.getPomFile() );
-
-        File parentPomFile = result.getRawModel( result.getModelIds().get( 1 ) ).getPomFile();
-        project.setParentFile( parentPomFile );
-
-        project.setParent( projects.get( result.getModelIds().get( 1 ) ) );
+        Parent p = model.getParent();
+        if ( p != null )
+        {
+            project.setParentArtifact( repositorySystem.createProjectArtifact( p.getGroupId(), p.getArtifactId(),
+                                                                               p.getVersion() ) );
+            // org.apache.maven.its.mng4834:parent:0.1
+            String parentModelId = result.getModelIds().get( 1 );
+            File parentPomFile = result.getRawModel( parentModelId ).getPomFile();
+            MavenProject parent = projects.get( parentModelId );
+            if ( parent == null )
+            {
+                //
+                // At this point the DefaultModelBuildingListener has fired and it populates the
+                // remote repositories with those found in the pom.xml, along with the existing externally
+                // defined repositories.
+                //
+                projectBuildingRequest.setRemoteRepositories( project.getRemoteArtifactRepositories() );
+                if ( parentPomFile != null )
+                {
+                    project.setParentFile( parentPomFile );
+                    try
+                    {
+                        parent = build( parentPomFile, projectBuildingRequest ).getProject();
+                    }
+                    catch ( ProjectBuildingException e )
+                    {
+                        // MNG-4488 where let invalid parents slide on by
+                        logger.warn( "Failed to build parent project for " + project.getId() );
+                    }
+                }
+                else
+                {
+                    Artifact parentArtifact = project.getParentArtifact();
+                    try
+                    {
+                        parent = build( parentArtifact, projectBuildingRequest ).getProject();
+                    }
+                    catch ( ProjectBuildingException e )
+                    {
+                        // MNG-4488 where let invalid parents slide on by
+                        logger.warn( "Failed to build parent project for " + project.getId() );
+                    }
+                }
+            }
+            project.setParent( parent );
+        }
 
         Artifact projectArtifact =
             repositorySystem.createArtifact( project.getGroupId(), project.getArtifactId(), project.getVersion(), null,
@@ -630,9 +731,149 @@ public class DefaultProjectBuilder
             ModelProblem problem =
                 new DefaultModelProblem( "Detected profiles.xml alongside " + modelId
                     + ", this file is no longer supported and was ignored" + ", please use the settings.xml instead",
-                                         ModelProblem.Severity.WARNING, model, -1, -1, null );
+                                         ModelProblem.Severity.WARNING, ModelProblem.Version.V30, model, -1, -1, null );
             result.getProblems().add( problem );
         }
+
+        //
+        // All the parts that were taken out of MavenProject for Maven 4.0.0
+        //
+
+        project.setProjectBuildingRequest( projectBuildingRequest );
+
+        // pluginArtifacts
+        Set<Artifact> pluginArtifacts = new HashSet<Artifact>();
+        for ( Plugin plugin : project.getBuildPlugins() )
+        {
+            Artifact artifact = repositorySystem.createPluginArtifact( plugin );
+
+            if ( artifact != null )
+            {
+                pluginArtifacts.add( artifact );
+            }
+        }
+        project.setPluginArtifacts( pluginArtifacts );
+
+        // reportArtifacts
+        Set<Artifact> reportArtifacts = new HashSet<Artifact>();
+        for ( ReportPlugin report : project.getReportPlugins() )
+        {
+            Plugin pp = new Plugin();
+            pp.setGroupId( report.getGroupId() );
+            pp.setArtifactId( report.getArtifactId() );
+            pp.setVersion( report.getVersion() );
+
+            Artifact artifact = repositorySystem.createPluginArtifact( pp );
+
+            if ( artifact != null )
+            {
+                reportArtifacts.add( artifact );
+            }
+        }
+        project.setReportArtifacts( reportArtifacts );
+
+        // extensionArtifacts
+        Set<Artifact> extensionArtifacts = new HashSet<Artifact>();
+        List<Extension> extensions = project.getBuildExtensions();
+        if ( extensions != null )
+        {
+            for ( Extension ext : extensions )
+            {
+                String version;
+                if ( StringUtils.isEmpty( ext.getVersion() ) )
+                {
+                    version = "RELEASE";
+                }
+                else
+                {
+                    version = ext.getVersion();
+                }
+
+                Artifact artifact =
+                    repositorySystem.createArtifact( ext.getGroupId(), ext.getArtifactId(), version, null, "jar" );
+
+                if ( artifact != null )
+                {
+                    extensionArtifacts.add( artifact );
+                }
+            }
+        }
+        project.setExtensionArtifacts( extensionArtifacts );
+
+        // managedVersionMap
+        Map<String, Artifact> map = null;
+        if ( repositorySystem != null )
+        {
+            List<Dependency> deps;
+            DependencyManagement dependencyManagement = project.getDependencyManagement();
+            if ( ( dependencyManagement != null ) && ( ( deps = dependencyManagement.getDependencies() ) != null )
+                && ( deps.size() > 0 ) )
+            {
+                map = new HashMap<String, Artifact>();
+                for ( Dependency d : dependencyManagement.getDependencies() )
+                {
+                    Artifact artifact = repositorySystem.createDependencyArtifact( d );
+
+                    if ( artifact != null )
+                    {
+                        map.put( d.getManagementKey(), artifact );
+                    }
+                }
+            }
+            else
+            {
+                map = Collections.emptyMap();
+            }
+        }
+        project.setManagedVersionMap( map );
+
+        // release artifact repository
+        if ( project.getDistributionManagement() != null
+                        && project.getDistributionManagement().getRepository() != null )
+        {
+            try
+            {
+                DeploymentRepository r = project.getDistributionManagement().getRepository();
+                if ( !StringUtils.isEmpty( r.getId() ) && !StringUtils.isEmpty( r.getUrl() ) )
+                {
+                    ArtifactRepository repo = repositorySystem.buildArtifactRepository( r );
+                    repositorySystem.injectProxy( projectBuildingRequest.getRepositorySession(),
+                                                  Arrays.asList( repo ) );
+                    repositorySystem.injectAuthentication( projectBuildingRequest.getRepositorySession(),
+                                                           Arrays.asList( repo ) );
+                    project.setReleaseArtifactRepository( repo );
+                }
+            }
+            catch ( InvalidRepositoryException e )
+            {
+                throw new IllegalStateException( "Failed to create release distribution repository for "
+                    + project.getId(), e );
+            }
+        }
+
+        // snapshot artifact repository
+        if ( project.getDistributionManagement() != null
+            && project.getDistributionManagement().getSnapshotRepository() != null )
+        {
+            try
+            {
+                DeploymentRepository r = project.getDistributionManagement().getSnapshotRepository();
+                if ( !StringUtils.isEmpty( r.getId() ) && !StringUtils.isEmpty( r.getUrl() ) )
+                {
+                    ArtifactRepository repo = repositorySystem.buildArtifactRepository( r );
+                    repositorySystem.injectProxy( projectBuildingRequest.getRepositorySession(),
+                                                  Arrays.asList( repo ) );
+                    repositorySystem.injectAuthentication( projectBuildingRequest.getRepositorySession(),
+                                                           Arrays.asList( repo ) );
+                    project.setSnapshotArtifactRepository( repo );
+                }
+            }
+            catch ( InvalidRepositoryException e )
+            {
+                throw new IllegalStateException( "Failed to create snapshot distribution repository for "
+                    + project.getId(), e );
+            }
+        }
     }
 
     private String findProfilesXml( ModelBuildingResult result, Map<File, Boolean> profilesXmls )
@@ -650,10 +891,10 @@ public class DefaultProjectBuilder
             Boolean profilesXml = profilesXmls.get( basedir );
             if ( profilesXml == null )
             {
-                profilesXml = Boolean.valueOf( new File( basedir, "profiles.xml" ).exists() );
+                profilesXml = new File( basedir, "profiles.xml" ).exists();
                 profilesXmls.put( basedir, profilesXml );
             }
-            if ( profilesXml.booleanValue() )
+            if ( profilesXml )
             {
                 return modelId;
             }
@@ -673,13 +914,10 @@ public class DefaultProjectBuilder
 
         public final ReactorModelPool modelPool;
 
-        public final ReactorModelCache modelCache;
-
-        public InternalConfig( ProjectBuildingRequest request, ReactorModelPool modelPool, ReactorModelCache modelCache )
+        InternalConfig( ProjectBuildingRequest request, ReactorModelPool modelPool )
         {
             this.request = request;
             this.modelPool = modelPool;
-            this.modelCache = modelCache;
             session =
                 LegacyLocalRepositoryManager.overlay( request.getLocalRepository(), request.getRepositorySession(),
                                                       repoSystem );
