@@ -52,7 +52,6 @@ import org.apache.maven.artifact.versioning.ArtifactVersion;
 import org.apache.maven.artifact.versioning.DefaultArtifactVersion;
 import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
 import org.apache.maven.artifact.versioning.VersionRange;
-import org.apache.maven.execution.MavenSession;
 import org.apache.maven.model.Dependency;
 import org.apache.maven.model.DependencyManagement;
 import org.apache.maven.model.DistributionManagement;
@@ -69,6 +68,7 @@ import org.apache.maven.project.ProjectBuilder;
 import org.apache.maven.project.ProjectBuildingException;
 import org.apache.maven.project.ProjectBuildingRequest;
 import org.apache.maven.properties.internal.EnvironmentUtils;
+import org.apache.maven.properties.internal.SystemProperties;
 import org.apache.maven.repository.legacy.metadata.DefaultMetadataResolutionRequest;
 import org.apache.maven.repository.legacy.metadata.MetadataResolutionRequest;
 import org.codehaus.plexus.PlexusContainer;
@@ -76,9 +76,9 @@ import org.codehaus.plexus.component.annotations.Component;
 import org.codehaus.plexus.component.annotations.Requirement;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
 import org.codehaus.plexus.logging.Logger;
-import org.sonatype.aether.RepositorySystemSession;
-import org.sonatype.aether.repository.RepositoryPolicy;
-import org.sonatype.aether.transfer.ArtifactNotFoundException;
+import org.eclipse.aether.RepositorySystemSession;
+import org.eclipse.aether.repository.RepositoryPolicy;
+import org.eclipse.aether.transfer.ArtifactNotFoundException;
 
 /**
  * @author Jason van Zyl
@@ -158,13 +158,11 @@ public class MavenMetadataSource
             cache.get( artifact, request.isResolveManagedVersions(), request.getLocalRepository(),
                        request.getRemoteRepositories() );
 
-        if ( cached != null )
+        if ( cached != null
+        // if the POM has no file, we cached a missing artifact, only return the cached data if no update forced
+            && ( !request.isForceUpdate() || hasFile( cached.getPomArtifact() ) ) )
         {
-            // if the POM has no file, we cached a missing artifact, only return the cached data if no update forced
-            if ( !request.isForceUpdate() || hasFile( cached.getPomArtifact() ) )
-            {
-                return cached;
-            }
+            return cached;
         }
 
         List<Dependency> dependencies;
@@ -202,7 +200,7 @@ public class MavenMetadataSource
 
             if ( rel.project == null )
             {
-                // When this happens we have a Maven 1.x POM, or some invalid POM. 
+                // When this happens we have a Maven 1.x POM, or some invalid POM.
                 // It should have never found its way into Maven 2.x repository but it did.
                 dependencies = Collections.emptyList();
             }
@@ -447,7 +445,7 @@ public class MavenMetadataSource
 
     public List<ArtifactVersion> retrieveAvailableVersionsFromDeploymentRepository( Artifact artifact,
                                                                                     ArtifactRepository localRepository,
-                                                                                    ArtifactRepository deploymentRepository )
+                                                                              ArtifactRepository deploymentRepository )
         throws ArtifactMetadataRetrievalException
     {
         RepositoryMetadata metadata = new ArtifactRepositoryMetadata( artifact );
@@ -735,12 +733,8 @@ public class MavenMetadataSource
         {
             return true;
         }
-        if ( e.getCause() instanceof org.sonatype.aether.resolution.ArtifactResolutionException
-            && e.getCause().getCause() instanceof ArtifactNotFoundException )
-        {
-            return true;
-        }
-        return false;
+        return e.getCause() instanceof org.eclipse.aether.resolution.ArtifactResolutionException
+            && e.getCause().getCause() instanceof ArtifactNotFoundException;
     }
 
     private boolean isNonTransferrablePom( Exception e )
@@ -749,12 +743,8 @@ public class MavenMetadataSource
         {
             return true;
         }
-        if ( e.getCause() instanceof org.sonatype.aether.resolution.ArtifactResolutionException
-            && !( e.getCause().getCause() instanceof ArtifactNotFoundException ) )
-        {
-            return true;
-        }
-        return false;
+        return e.getCause() instanceof org.eclipse.aether.resolution.ArtifactResolutionException
+            && !( e.getCause().getCause() instanceof ArtifactNotFoundException );
     }
 
     private Properties getSystemProperties()
@@ -763,7 +753,7 @@ public class MavenMetadataSource
 
         EnvironmentUtils.addEnvVars( props );
 
-        props.putAll( System.getProperties() );
+        SystemProperties.addSystemProperties( props );
 
         return props;
     }
