@@ -18,11 +18,17 @@ package org.apache.maven.lifecycle;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 
 import org.apache.maven.AbstractCoreMavenComponentTestCase;
 import org.apache.maven.exception.ExceptionHandler;
 import org.apache.maven.execution.MavenSession;
+import org.apache.maven.execution.MojoExecutionEvent;
+import org.apache.maven.execution.MojoExecutionListener;
+import org.apache.maven.execution.ProjectDependencyGraph;
+import org.apache.maven.execution.ProjectExecutionEvent;
+import org.apache.maven.execution.ProjectExecutionListener;
 import org.apache.maven.lifecycle.internal.DefaultLifecycleTaskSegmentCalculator;
 import org.apache.maven.lifecycle.internal.ExecutionPlanItem;
 import org.apache.maven.lifecycle.internal.LifecycleExecutionPlanCalculator;
@@ -32,8 +38,10 @@ import org.apache.maven.lifecycle.internal.MojoDescriptorCreator;
 import org.apache.maven.lifecycle.internal.TaskSegment;
 import org.apache.maven.model.Plugin;
 import org.apache.maven.plugin.MojoExecution;
+import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoNotFoundException;
 import org.apache.maven.plugin.descriptor.MojoDescriptor;
+import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.component.annotations.Requirement;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
 
@@ -205,14 +213,14 @@ public class LifecycleExecutorTest
         //[05] modello:xpp3-writer
         //[06] modello:java
         //[07] modello:xpp3-reader
-        //[08] plugin:descriptor        
+        //[08] plugin:descriptor
         //[09] resources:resources
         //[10] compiler:compile
         //[11] resources:testResources
         //[12] compiler:testCompile
         //[13] surefire:test
         //[14] jar:jar
-        //[15] plugin:addPluginArtifactMetadata        
+        //[15] plugin:addPluginArtifactMetadata
         //[16] install:install
         //
 
@@ -225,9 +233,9 @@ public class LifecycleExecutorTest
         assertEquals( "it:xpp3-writer", executions.get( 4 ).getMojoDescriptor().getFullGoalName() );
         assertEquals( "it:java", executions.get( 5 ).getMojoDescriptor().getFullGoalName() );
         assertEquals( "it:xpp3-reader", executions.get( 6 ).getMojoDescriptor().getFullGoalName() );
-        assertEquals( "plugin:descriptor", executions.get( 7 ).getMojoDescriptor().getFullGoalName() );
-        assertEquals( "resources:resources", executions.get( 8 ).getMojoDescriptor().getFullGoalName() );
-        assertEquals( "compiler:compile", executions.get( 9 ).getMojoDescriptor().getFullGoalName() );
+        assertEquals( "resources:resources", executions.get( 7 ).getMojoDescriptor().getFullGoalName() );
+        assertEquals( "compiler:compile", executions.get( 8 ).getMojoDescriptor().getFullGoalName() );
+        assertEquals( "plugin:descriptor", executions.get( 9 ).getMojoDescriptor().getFullGoalName() );
         assertEquals( "resources:testResources", executions.get( 10 ).getMojoDescriptor().getFullGoalName() );
         assertEquals( "compiler:testCompile", executions.get( 11 ).getMojoDescriptor().getFullGoalName() );
         assertEquals( "surefire:test", executions.get( 12 ).getMojoDescriptor().getFullGoalName() );
@@ -380,4 +388,166 @@ public class LifecycleExecutorTest
         assertEquals("1.0", execution.getConfiguration().getChild( "version" ).getAttribute( "default-value" ));
     }
 
+    public void testExecutionListeners()
+        throws Exception
+    {
+        final File pom = getProject( "project-basic" );
+        final MavenSession session = createMavenSession( pom );
+        session.setProjectDependencyGraph( new ProjectDependencyGraph()
+        {
+            public List<MavenProject> getUpstreamProjects( MavenProject project, boolean transitive )
+            {
+                return Collections.emptyList();
+            }
+
+            public List<MavenProject> getSortedProjects()
+            {
+                return Collections.singletonList( session.getCurrentProject() );
+            }
+
+            public List<MavenProject> getDownstreamProjects( MavenProject project, boolean transitive )
+            {
+                return Collections.emptyList();
+            }
+            
+            public java.util.List<MavenProject> getAllSortedProjects()
+            {
+                return Collections.emptyList();
+            }
+        } );
+
+        final List<String> log = new ArrayList<String>();
+
+        MojoExecutionListener mojoListener = new MojoExecutionListener()
+        {
+            public void beforeMojoExecution( MojoExecutionEvent event )
+                throws MojoExecutionException
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNotNull( event.getExecution() );
+                assertNotNull( event.getMojo() );
+                assertNull( event.getCause() );
+
+                log.add( "beforeMojoExecution " + event.getProject().getArtifactId() + ":"
+                    + event.getExecution().getExecutionId() );
+            }
+
+            public void afterMojoExecutionSuccess( MojoExecutionEvent event )
+                throws MojoExecutionException
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNotNull( event.getExecution() );
+                assertNotNull( event.getMojo() );
+                assertNull( event.getCause() );
+
+                log.add( "afterMojoExecutionSuccess " + event.getProject().getArtifactId() + ":"
+                    + event.getExecution().getExecutionId() );
+            }
+
+            public void afterExecutionFailure( MojoExecutionEvent event )
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNotNull( event.getExecution() );
+                assertNotNull( event.getMojo() );
+                assertNotNull( event.getCause() );
+
+                log.add( "afterExecutionFailure " + event.getProject().getArtifactId() + ":"
+                    + event.getExecution().getExecutionId() );
+            }
+        };
+        ProjectExecutionListener projectListener = new ProjectExecutionListener()
+        {
+            public void beforeProjectExecution( ProjectExecutionEvent event )
+                throws LifecycleExecutionException
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNull( event.getExecutionPlan() );
+                assertNull( event.getCause() );
+
+                log.add( "beforeProjectExecution " + event.getProject().getArtifactId() );
+            }
+
+            public void beforeProjectLifecycleExecution( ProjectExecutionEvent event )
+                throws LifecycleExecutionException
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNotNull( event.getExecutionPlan() );
+                assertNull( event.getCause() );
+
+                log.add( "beforeProjectLifecycleExecution " + event.getProject().getArtifactId() );
+            }
+
+            public void afterProjectExecutionSuccess( ProjectExecutionEvent event )
+                throws LifecycleExecutionException
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNotNull( event.getExecutionPlan() );
+                assertNull( event.getCause() );
+
+                log.add( "afterProjectExecutionSuccess " + event.getProject().getArtifactId() );
+            }
+
+            public void afterProjectExecutionFailure( ProjectExecutionEvent event )
+            {
+                assertNotNull( event.getSession() );
+                assertNotNull( event.getProject() );
+                assertNull( event.getExecutionPlan() );
+                assertNotNull( event.getCause() );
+
+                log.add( "afterProjectExecutionFailure " + event.getProject().getArtifactId() );
+            }
+        };
+        lookup( DelegatingProjectExecutionListener.class ).addProjectExecutionListener( projectListener );
+        lookup( DelegatingMojoExecutionListener.class ).addMojoExecutionListener( mojoListener );
+
+        try
+        {
+            lifecycleExecutor.execute( session );
+        }
+        finally
+        {
+            lookup( DelegatingProjectExecutionListener.class ).removeProjectExecutionListener( projectListener );
+            lookup( DelegatingMojoExecutionListener.class ).removeMojoExecutionListener( mojoListener );
+        }
+
+        List<String> expectedLog = Arrays.asList( "beforeProjectExecution project-basic", //
+                                                  "beforeProjectLifecycleExecution project-basic", //
+                                                  "beforeMojoExecution project-basic:default-resources", //
+                                                  "afterMojoExecutionSuccess project-basic:default-resources", //
+                                                  "beforeMojoExecution project-basic:default-compile", //
+                                                  "afterMojoExecutionSuccess project-basic:default-compile", //
+                                                  "beforeMojoExecution project-basic:default-testResources", //
+                                                  "afterMojoExecutionSuccess project-basic:default-testResources", //
+                                                  "beforeMojoExecution project-basic:default-testCompile", //
+                                                  "afterMojoExecutionSuccess project-basic:default-testCompile", //
+                                                  "beforeMojoExecution project-basic:default-test", //
+                                                  "afterMojoExecutionSuccess project-basic:default-test", //
+                                                  "beforeMojoExecution project-basic:default-jar", //
+                                                  "afterMojoExecutionSuccess project-basic:default-jar", //
+                                                  "afterProjectExecutionSuccess project-basic" //
+        );
+
+        assertEventLog( expectedLog, log );
+    }
+
+    private static void assertEventLog( List<String> expectedList, List<String> actualList )
+    {
+        assertEquals( toString( expectedList ), toString( actualList ) );
+    }
+
+    private static String toString( List<String> lines )
+    {
+        StringBuilder sb = new StringBuilder();
+        for ( String line : lines )
+        {
+            sb.append( line ).append( '\n' );
+        }
+        return sb.toString();
+    }
 }
